﻿/**
 * This specialized UIEvent responds to input key events generated by the system.  Unlike other event classes, this event
 * class executes different actions depending on the input key that caused the event to be activated.
 *
 * Widgets must contain a UIEvent_ProcessInput in their list of implemented events in order to process input events.
 *
 * Copyright 1998-2008 Epic Games, Inc. All Rights Reserved.
 */
class UIEvent_ProcessInput extends UIEvent
	native(inherit)
	DependsOn(UIRoot)
	transient;

/**
 * Contains the list of input keys which are currently active, along with the optional UIActions that should be
 * activated when the input event is received.  Initialized when the owning widget is initialized, and updated
 * as UIStates add and remove input keys so that actions here always represent the set of actions the widget can
 * currently respond to.
 *
 * Input key events that are handled natively will typically not have an associated UIAction.  For those that do, the
 * native code will be executed first, then the UIAction will be activated.
 */
var	native	transient MultiMap_Mirror		ActionMap{TMultiMap<FName,FInputKeyAction>};



/**
 * Determines whether this class should be displayed in the list of available ops in the UI's kismet editor.
 *
 * @param	TargetObject	the widget that this SequenceObject would be attached to.
 *
 * @return	TRUE if this sequence object should be available for use in the UI kismet editor
 */
event bool IsValidUISequenceObject( optional UIScreenObject TargetObject )
{
	return false;
}

DefaultProperties
{
	ObjName="Handle Input"

	// don't add input events to the widget's sequences - these will be handled differently
	bShouldRegisterEvent=false
	bPropagateEvent=false
}
