﻿
/**
 * Copyright 1998-2008 Epic Games, Inc. All Rights Reserved.
 */

/** The class that writes the DM stats */

class UTLeaderboardWriteVehiclesDM extends UTLeaderboardWriteBase;

`include(UTStats.uci)

//Copies all relevant PRI game stats into the Properties struct of the OnlineStatsWrite
//There can be many more stats in the PRI than what is in the Properties table (on Xbox for example)
//If the Properties table does not contain the entry, the data is not written
function CopyAllStats(UTPlayerReplicationInfo PRI)
{
	local IntStat tempIntStat;
	local TimeStat tempTimeStat;

	//Kill stats (collision/runover)
	foreach PRI.KillStats(tempIntStat)
	{
		SetIntStatFromMapping(tempIntStat.StatName, tempIntStat.StatValue);
	}

	//Death stats (collision/runover)
	foreach PRI.DeathStats(tempIntStat)
	{
		SetIntStatFromMapping(tempIntStat.StatName, tempIntStat.StatValue);
	}

	//Number of vehicles destroyed
	foreach PRI.VehicleKillStats(tempIntStat)
	{
		SetIntStatFromMapping(tempIntStat.StatName, tempIntStat.StatValue);
	}

	//Time driving in a vehicle
	foreach PRI.DrivingStats(tempTimeStat)
	{
		SetIntStatFromMapping(tempTimeStat.StatName, int(tempTimeStat.TotalTime));
	}

	Super.CopyAllStats(PRI);
}

defaultproperties
{
	// Sort the leaderboard by this property
RatingId=PROPERTY_LEADERBOARDRATING

	// Views being written to depending on type of match (ranked or player)
ViewIds=(STATS_VIEW_DM_VEHICLES_ALLTIME)
PureViewIds=(STATS_VIEW_DM_VEHICLES_RANKED_ALLTIME)
ArbitratedViewIds=(STATS_VIEW_DM_VEHICLES_RANKED_ALLTIME)

	//All properties for the given table and their types
Properties.Add((PropertyId=`PROPERTY_DRIVING_UTVEHICLE_CICADA_CONTENT,Data=(Type=SDT_Int32,Value1=0)))
Properties.Add((PropertyId=`PROPERTY_DRIVING_UTVEHICLE_DARKWALKER_CONTENT,Data=(Type=SDT_Int32,Value1=0)))
Properties.Add((PropertyId=`PROPERTY_DRIVING_UTVEHICLE_FURY_CONTENT,Data=(Type=SDT_Int32,Value1=0)))
Properties.Add((PropertyId=`PROPERTY_DRIVING_UTVEHICLE_GOLIATH_CONTENT,Data=(Type=SDT_Int32,Value1=0)))
Properties.Add((PropertyId=`PROPERTY_DRIVING_UTVEHICLE_HELLBENDER_CONTENT,Data=(Type=SDT_Int32,Value1=0)))
Properties.Add((PropertyId=`PROPERTY_DRIVING_UTVEHICLE_HOVERBOARD,Data=(Type=SDT_Int32,Value1=0)))
Properties.Add((PropertyId=`PROPERTY_DRIVING_UTVEHICLE_LEVIATHAN_CONTENT,Data=(Type=SDT_Int32,Value1=0)))
Properties.Add((PropertyId=`PROPERTY_DRIVING_UTVEHICLE_MANTA_CONTENT,Data=(Type=SDT_Int32,Value1=0)))
Properties.Add((PropertyId=`PROPERTY_DRIVING_UTVEHICLE_NEMESIS,Data=(Type=SDT_Int32,Value1=0)))
Properties.Add((PropertyId=`PROPERTY_DRIVING_UTVEHICLE_NIGHTSHADE_CONTENT,Data=(Type=SDT_Int32,Value1=0)))
Properties.Add((PropertyId=`PROPERTY_DRIVING_UTVEHICLE_PALADIN,Data=(Type=SDT_Int32,Value1=0)))
Properties.Add((PropertyId=`PROPERTY_DRIVING_UTVEHICLE_RAPTOR_CONTENT,Data=(Type=SDT_Int32,Value1=0)))
Properties.Add((PropertyId=`PROPERTY_DRIVING_UTVEHICLE_SCAVENGER_CONTENT,Data=(Type=SDT_Int32,Value1=0)))
Properties.Add((PropertyId=`PROPERTY_DRIVING_UTVEHICLE_SCORPION_CONTENT,Data=(Type=SDT_Int32,Value1=0)))
Properties.Add((PropertyId=`PROPERTY_DRIVING_UTVEHICLE_SPMA_CONTENT,Data=(Type=SDT_Int32,Value1=0)))
Properties.Add((PropertyId=`PROPERTY_DRIVING_UTVEHICLE_STEALTHBENDER_CONTENT,Data=(Type=SDT_Int32,Value1=0)))
Properties.Add((PropertyId=`PROPERTY_DRIVING_UTVEHICLE_TURRET,Data=(Type=SDT_Int32,Value1=0)))
Properties.Add((PropertyId=`PROPERTY_DRIVING_UTVEHICLE_VIPER_CONTENT,Data=(Type=SDT_Int32,Value1=0)))
Properties.Add((PropertyId=`PROPERTY_VEHICLEKILL_UTVEHICLE_CICADA_CONTENT,Data=(Type=SDT_Int32,Value1=0)))
Properties.Add((PropertyId=`PROPERTY_VEHICLEKILL_UTVEHICLE_DARKWALKER_CONTENT,Data=(Type=SDT_Int32,Value1=0)))
Properties.Add((PropertyId=`PROPERTY_VEHICLEKILL_UTVEHICLE_FURY_CONTENT,Data=(Type=SDT_Int32,Value1=0)))
Properties.Add((PropertyId=`PROPERTY_VEHICLEKILL_UTVEHICLE_GOLIATH_CONTENT,Data=(Type=SDT_Int32,Value1=0)))
Properties.Add((PropertyId=`PROPERTY_VEHICLEKILL_UTVEHICLE_HELLBENDER_CONTENT,Data=(Type=SDT_Int32,Value1=0)))
Properties.Add((PropertyId=`PROPERTY_VEHICLEKILL_UTVEHICLE_HOVERBOARD,Data=(Type=SDT_Int32,Value1=0)))
Properties.Add((PropertyId=`PROPERTY_VEHICLEKILL_UTVEHICLE_LEVIATHAN_CONTENT,Data=(Type=SDT_Int32,Value1=0)))
Properties.Add((PropertyId=`PROPERTY_VEHICLEKILL_UTVEHICLE_MANTA_CONTENT,Data=(Type=SDT_Int32,Value1=0)))
Properties.Add((PropertyId=`PROPERTY_VEHICLEKILL_UTVEHICLE_NEMESIS,Data=(Type=SDT_Int32,Value1=0)))
Properties.Add((PropertyId=`PROPERTY_VEHICLEKILL_UTVEHICLE_NIGHTSHADE_CONTENT,Data=(Type=SDT_Int32,Value1=0)))
Properties.Add((PropertyId=`PROPERTY_VEHICLEKILL_UTVEHICLE_PALADIN,Data=(Type=SDT_Int32,Value1=0)))
Properties.Add((PropertyId=`PROPERTY_VEHICLEKILL_UTVEHICLE_RAPTOR_CONTENT,Data=(Type=SDT_Int32,Value1=0)))
Properties.Add((PropertyId=`PROPERTY_VEHICLEKILL_UTVEHICLE_SCAVENGER_CONTENT,Data=(Type=SDT_Int32,Value1=0)))
Properties.Add((PropertyId=`PROPERTY_VEHICLEKILL_UTVEHICLE_SCORPION_CONTENT,Data=(Type=SDT_Int32,Value1=0)))
Properties.Add((PropertyId=`PROPERTY_VEHICLEKILL_UTVEHICLE_SPMA_CONTENT,Data=(Type=SDT_Int32,Value1=0)))
Properties.Add((PropertyId=`PROPERTY_VEHICLEKILL_UTVEHICLE_STEALTHBENDER_CONTENT,Data=(Type=SDT_Int32,Value1=0)))
Properties.Add((PropertyId=`PROPERTY_VEHICLEKILL_UTVEHICLE_TURRET,Data=(Type=SDT_Int32,Value1=0)))
Properties.Add((PropertyId=`PROPERTY_VEHICLEKILL_UTVEHICLE_VIPER_CONTENT,Data=(Type=SDT_Int32,Value1=0)))

	//The mappings for every PRI in game stat to every Online stat regardless
	//of whether or not we're writing it out this session
StatNameToStatIdMapping.Add((StatName=DRIVING_UTVEHICLE_CICADA_CONTENT,Id=`PROPERTY_DRIVING_UTVEHICLE_CICADA_CONTENT))
StatNameToStatIdMapping.Add((StatName=DRIVING_UTVEHICLE_DARKWALKER_CONTENT,Id=`PROPERTY_DRIVING_UTVEHICLE_DARKWALKER_CONTENT))
StatNameToStatIdMapping.Add((StatName=DRIVING_UTVEHICLE_FURY_CONTENT,Id=`PROPERTY_DRIVING_UTVEHICLE_FURY_CONTENT))
StatNameToStatIdMapping.Add((StatName=DRIVING_UTVEHICLE_GOLIATH_CONTENT,Id=`PROPERTY_DRIVING_UTVEHICLE_GOLIATH_CONTENT))
StatNameToStatIdMapping.Add((StatName=DRIVING_UTVEHICLE_HELLBENDER_CONTENT,Id=`PROPERTY_DRIVING_UTVEHICLE_HELLBENDER_CONTENT))
StatNameToStatIdMapping.Add((StatName=DRIVING_UTVEHICLE_HOVERBOARD,Id=`PROPERTY_DRIVING_UTVEHICLE_HOVERBOARD))
StatNameToStatIdMapping.Add((StatName=DRIVING_UTVEHICLE_LEVIATHAN_CONTENT,Id=`PROPERTY_DRIVING_UTVEHICLE_LEVIATHAN_CONTENT))
StatNameToStatIdMapping.Add((StatName=DRIVING_UTVEHICLE_MANTA_CONTENT,Id=`PROPERTY_DRIVING_UTVEHICLE_MANTA_CONTENT))
StatNameToStatIdMapping.Add((StatName=DRIVING_UTVEHICLE_NEMESIS,Id=`PROPERTY_DRIVING_UTVEHICLE_NEMESIS))
StatNameToStatIdMapping.Add((StatName=DRIVING_UTVEHICLE_NIGHTSHADE_CONTENT,Id=`PROPERTY_DRIVING_UTVEHICLE_NIGHTSHADE_CONTENT))
StatNameToStatIdMapping.Add((StatName=DRIVING_UTVEHICLE_PALADIN,Id=`PROPERTY_DRIVING_UTVEHICLE_PALADIN))
StatNameToStatIdMapping.Add((StatName=DRIVING_UTVEHICLE_RAPTOR_CONTENT,Id=`PROPERTY_DRIVING_UTVEHICLE_RAPTOR_CONTENT))
StatNameToStatIdMapping.Add((StatName=DRIVING_UTVEHICLE_SCAVENGER_CONTENT,Id=`PROPERTY_DRIVING_UTVEHICLE_SCAVENGER_CONTENT))
StatNameToStatIdMapping.Add((StatName=DRIVING_UTVEHICLE_SCORPION_CONTENT,Id=`PROPERTY_DRIVING_UTVEHICLE_SCORPION_CONTENT))
StatNameToStatIdMapping.Add((StatName=DRIVING_UTVEHICLE_SPMA_CONTENT,Id=`PROPERTY_DRIVING_UTVEHICLE_SPMA_CONTENT))
StatNameToStatIdMapping.Add((StatName=DRIVING_UTVEHICLE_STEALTHBENDER_CONTENT,Id=`PROPERTY_DRIVING_UTVEHICLE_STEALTHBENDER_CONTENT))
StatNameToStatIdMapping.Add((StatName=DRIVING_UTVEHICLE_TURRET,Id=`PROPERTY_DRIVING_UTVEHICLE_TURRET))
StatNameToStatIdMapping.Add((StatName=DRIVING_UTVEHICLE_VIPER_CONTENT,Id=`PROPERTY_DRIVING_UTVEHICLE_VIPER_CONTENT))
StatNameToStatIdMapping.Add((StatName=VEHICLEKILL_UTVEHICLE_CICADA_CONTENT,Id=`PROPERTY_VEHICLEKILL_UTVEHICLE_CICADA_CONTENT))
StatNameToStatIdMapping.Add((StatName=VEHICLEKILL_UTVEHICLE_DARKWALKER_CONTENT,Id=`PROPERTY_VEHICLEKILL_UTVEHICLE_DARKWALKER_CONTENT))
StatNameToStatIdMapping.Add((StatName=VEHICLEKILL_UTVEHICLE_FURY_CONTENT,Id=`PROPERTY_VEHICLEKILL_UTVEHICLE_FURY_CONTENT))
StatNameToStatIdMapping.Add((StatName=VEHICLEKILL_UTVEHICLE_GOLIATH_CONTENT,Id=`PROPERTY_VEHICLEKILL_UTVEHICLE_GOLIATH_CONTENT))
StatNameToStatIdMapping.Add((StatName=VEHICLEKILL_UTVEHICLE_HELLBENDER_CONTENT,Id=`PROPERTY_VEHICLEKILL_UTVEHICLE_HELLBENDER_CONTENT))
StatNameToStatIdMapping.Add((StatName=VEHICLEKILL_UTVEHICLE_HOVERBOARD,Id=`PROPERTY_VEHICLEKILL_UTVEHICLE_HOVERBOARD))
StatNameToStatIdMapping.Add((StatName=VEHICLEKILL_UTVEHICLE_LEVIATHAN_CONTENT,Id=`PROPERTY_VEHICLEKILL_UTVEHICLE_LEVIATHAN_CONTENT))
StatNameToStatIdMapping.Add((StatName=VEHICLEKILL_UTVEHICLE_MANTA_CONTENT,Id=`PROPERTY_VEHICLEKILL_UTVEHICLE_MANTA_CONTENT))
StatNameToStatIdMapping.Add((StatName=VEHICLEKILL_UTVEHICLE_NEMESIS,Id=`PROPERTY_VEHICLEKILL_UTVEHICLE_NEMESIS))
StatNameToStatIdMapping.Add((StatName=VEHICLEKILL_UTVEHICLE_NIGHTSHADE_CONTENT,Id=`PROPERTY_VEHICLEKILL_UTVEHICLE_NIGHTSHADE_CONTENT))
StatNameToStatIdMapping.Add((StatName=VEHICLEKILL_UTVEHICLE_PALADIN,Id=`PROPERTY_VEHICLEKILL_UTVEHICLE_PALADIN))
StatNameToStatIdMapping.Add((StatName=VEHICLEKILL_UTVEHICLE_RAPTOR_CONTENT,Id=`PROPERTY_VEHICLEKILL_UTVEHICLE_RAPTOR_CONTENT))
StatNameToStatIdMapping.Add((StatName=VEHICLEKILL_UTVEHICLE_SCAVENGER_CONTENT,Id=`PROPERTY_VEHICLEKILL_UTVEHICLE_SCAVENGER_CONTENT))
StatNameToStatIdMapping.Add((StatName=VEHICLEKILL_UTVEHICLE_SCORPION_CONTENT,Id=`PROPERTY_VEHICLEKILL_UTVEHICLE_SCORPION_CONTENT))
StatNameToStatIdMapping.Add((StatName=VEHICLEKILL_UTVEHICLE_SPMA_CONTENT,Id=`PROPERTY_VEHICLEKILL_UTVEHICLE_SPMA_CONTENT))
StatNameToStatIdMapping.Add((StatName=VEHICLEKILL_UTVEHICLE_STEALTHBENDER_CONTENT,Id=`PROPERTY_VEHICLEKILL_UTVEHICLE_STEALTHBENDER_CONTENT))
StatNameToStatIdMapping.Add((StatName=VEHICLEKILL_UTVEHICLE_TURRET,Id=`PROPERTY_VEHICLEKILL_UTVEHICLE_TURRET))
StatNameToStatIdMapping.Add((StatName=VEHICLEKILL_UTVEHICLE_VIPER_CONTENT,Id=`PROPERTY_VEHICLEKILL_UTVEHICLE_VIPER_CONTENT))

}
